/*
*  Copyright:    Copyright (c) 2001 Global InfoTek, Inc.
*  Author:       Cynthia Cicalese
*  Date:         December, 2001
*  Company:      Global InfoTek, Inc.
*  Description:  Sensor database graphical user interface
*
********************************************************************************
* This code is the exclusive property of Global InfoTek Inc. and may not
* be used for any purpose without the written permission of Global
* InfoTek, Inc.
********************************************************************************
* Revision History
*/

package tutorial.client;

import tutorial.Sensor;
import tutorial.shared.SensorDirectory;
import java.rmi.RemoteException;
import java.io.IOException;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Container;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.ActionEvent;
import java.sql.SQLException;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JOptionPane;
import javax.swing.Box;
import javax.swing.BoxLayout;

public class SensorGUI extends JFrame {

  private static final int WIDTH = 500;
  private static final int HEIGHT = 500;
  private static final int TEXTSIZE = 14;

  private SensorDirectory sensorDB;
  private JTextArea statusTextArea;

  public SensorGUI(SensorDirectory sensorDB) {
    this.sensorDB = sensorDB;
    buildGUI();
  }

  private void buildGUI() {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    setSize(new Dimension(WIDTH, HEIGHT));
    setTitle("Sensor Database");

    Font font = new Font("Monospaced", Font.BOLD, TEXTSIZE);

    Box content = new Box(BoxLayout.X_AXIS);
    Box leftContent = new Box(BoxLayout.Y_AXIS);
    content.add(leftContent);
    content.add(Box.createHorizontalStrut(20));
    Box rightContent = new Box(BoxLayout.Y_AXIS);
    content.add(rightContent);

    leftContent.add(Box.createVerticalStrut(10));

    JButton addSensorButton = new JButton("Add Sensor");
    addSensorButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        statusTextArea.setText("");
        addSensor();
      }
    });
    leftContent.add(addSensorButton);

    JButton removeSensorButton = new JButton("Remove Sensor");
    removeSensorButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        statusTextArea.setText("");
        removeSensor();
      }
    });
    leftContent.add(removeSensorButton);

    JButton setSensorActiveButton = new JButton("Set Sensor Active");
    setSensorActiveButton.addActionListener(
      new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        statusTextArea.setText("");
        setSensorActive();
      }
    });
    leftContent.add(setSensorActiveButton);

    JButton setSensorInactiveButton = new JButton("Set Sensor Inactive");
    setSensorInactiveButton.addActionListener(
      new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        statusTextArea.setText("");
        setSensorInactive();
      }
    });
    leftContent.add(setSensorInactiveButton);

    JButton getActiveSensorsButton = new JButton("Get Active Sensors");
    getActiveSensorsButton.addActionListener(
      new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        statusTextArea.setText("");
        getActiveSensors();
      }
    });
    leftContent.add(getActiveSensorsButton);

    JButton getInactiveSensorsButton = new JButton("Get Inactive Sensors");
    getInactiveSensorsButton.addActionListener(
      new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        statusTextArea.setText("");
        getInactiveSensors();
      }
    });
    leftContent.add(getInactiveSensorsButton);

    JButton getAllSensorsButton = new JButton("Get All Sensors");
    getAllSensorsButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        statusTextArea.setText("");
        getSensors();
      }
    });
    leftContent.add(getAllSensorsButton);

    rightContent.add(new JLabel("Status:"));

    statusTextArea = new JTextArea();
    statusTextArea.setFont(font);
    statusTextArea.setLineWrap(true);
    statusTextArea.setEditable(false);
    JScrollPane statusScrollPane = new JScrollPane();
    statusScrollPane.setPreferredSize(new Dimension(WIDTH, HEIGHT));
    statusScrollPane.getViewport().add(statusTextArea, null);
    rightContent.add(statusScrollPane);

    getContentPane().add(content);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent event) {
        System.exit(0);
      }
    });
  }

  private void addSensor() {
    String sensorName = JOptionPane.showInputDialog(getContentPane(),
      "What is the name of the sensor to add?", "Input",
      JOptionPane.QUESTION_MESSAGE);
    if (sensorName != null) {
      Object[] possibleValues = { "Active", "Inactive" };
      Object active = JOptionPane.showInputDialog(getContentPane(),
        "Is the sensor is active?", "Input", JOptionPane.QUESTION_MESSAGE,
        null, possibleValues, possibleValues[0]);
      if (active != null) {
        try {
          if (active == possibleValues[0]) {
            sensorDB.addSensor(sensorName, true);
            statusTextArea.setText("Added sensor " + sensorName +
              " (Active).");
          } else {
            sensorDB.addSensor(sensorName, false);
            statusTextArea.setText("Added sensor " + sensorName +
              " (Inactive).");
          }
        } catch (SQLException exc) {
          JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
            "Error", JOptionPane.ERROR_MESSAGE);
        } catch (RemoteException exc) {
          JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
            "Error", JOptionPane.ERROR_MESSAGE);
        }
      }
    }
  }

  private void removeSensor() {
    String sensorName = JOptionPane.showInputDialog(getContentPane(),
      "What is the name of the sensor to remove?", "Input",
      JOptionPane.QUESTION_MESSAGE);
    if (sensorName != null) {
      try {
        int num = sensorDB.removeSensor(sensorName);
        if (num == 0)
          JOptionPane.showMessageDialog(getContentPane(),
            "No sensors with name " + sensorName + " found.", "Error",
            JOptionPane.ERROR_MESSAGE);
        else statusTextArea.setText("Removed sensor " + sensorName + ".");
      } catch (SQLException exc) {
        JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
          "Error", JOptionPane.ERROR_MESSAGE);
      } catch (RemoteException exc) {
        JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
          "Error", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  private void setSensorActive() {
    String sensorName = JOptionPane.showInputDialog(getContentPane(),
      "What is the name of the sensor to make active?", "Input",
      JOptionPane.QUESTION_MESSAGE);
    if (sensorName != null) {
      try {
        int num = sensorDB.setActive(sensorName, true);
        if (num == 0)
          JOptionPane.showMessageDialog(getContentPane(),
            "No sensors with name " + sensorName + " found.", "Error",
            JOptionPane.ERROR_MESSAGE);
        else statusTextArea.setText("Made sensor " + sensorName + " active"
          + ".");
      } catch (SQLException exc) {
        JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
          "Error", JOptionPane.ERROR_MESSAGE);
      } catch (RemoteException exc) {
        JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
          "Error", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  private void setSensorInactive() {
    String sensorName = JOptionPane.showInputDialog(getContentPane(),
      "What is the name of the sensor to make inactive?", "Input",
      JOptionPane.QUESTION_MESSAGE);
    if (sensorName != null) {
      try {
        int num = sensorDB.setActive(sensorName, false);
        if (num == 0)
          JOptionPane.showMessageDialog(getContentPane(),
            "No sensors with name " + sensorName + " found.", "Error",
            JOptionPane.ERROR_MESSAGE);
        else statusTextArea.setText("Made sensor " + sensorName + " inactive"
          + ".");
      } catch (SQLException exc) {
        JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
          "Error", JOptionPane.ERROR_MESSAGE);
      } catch (RemoteException exc) {
        JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
          "Error", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  private void getActiveSensors() {
    try {
      String[] sensorNames = sensorDB.getActiveSensors();
      String text = "Active Sensors:\n";
      for (int i = 0; i < sensorNames.length; i++)
        text += sensorNames[i] + '\n';
      statusTextArea.setText(text);
      statusTextArea.setCaretPosition(0);
    } catch (SQLException exc) {
      JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(), "Error",
          JOptionPane.ERROR_MESSAGE);
    } catch (RemoteException exc) {
      JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
        "Error", JOptionPane.ERROR_MESSAGE);
    }
  }

  private void getInactiveSensors() {
    try {
      String[] sensorNames = sensorDB.getInactiveSensors();
      String text = "Inactive Sensors:\n";
      for (int i = 0; i < sensorNames.length; i++)
        text += sensorNames[i] + '\n';
      statusTextArea.setText(text);
      statusTextArea.setCaretPosition(0);
    } catch (SQLException exc) {
      JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(), "Error",
          JOptionPane.ERROR_MESSAGE);
    } catch (RemoteException exc) {
      JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
        "Error", JOptionPane.ERROR_MESSAGE);
    }
  }

  private void getSensors() {
    try {
      Sensor[] sensors = sensorDB.getSensors();
      String text = "All Sensors:\n";
      for (int i = 0; i < sensors.length; i++)
        text += "Name: " + sensors[i].name + ", Active: " +
          sensors[i].active + '\n';
      statusTextArea.setText(text);
      statusTextArea.setCaretPosition(0);
    } catch (SQLException exc) {
      JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(), "Error",
          JOptionPane.ERROR_MESSAGE);
    } catch (RemoteException exc) {
      JOptionPane.showMessageDialog(getContentPane(), exc.getMessage(),
        "Error", JOptionPane.ERROR_MESSAGE);
    }
  }
}
